package com.example.mobileapp.navigation

import android.annotation.SuppressLint
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Scaffold
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.currentBackStackEntryAsState
import androidx.navigation.compose.rememberNavController
import com.example.mobileapp.PreferencesManager
import com.example.mobileapp.db.AppDatabase
import com.example.mobileapp.models.User
import com.example.mobileapp.screens.AuthorizationScreen
import com.example.mobileapp.screens.CartScreen
import com.example.mobileapp.screens.CreateProductScreen
import com.example.mobileapp.screens.EditMenuScreen
import com.example.mobileapp.screens.EditProductScreen
import com.example.mobileapp.screens.ManagmentScreen
import com.example.mobileapp.screens.MenuScreen
import com.example.mobileapp.screens.OrderScreen
import com.example.mobileapp.screens.ProfileScreen
import com.example.mobileapp.screens.RegistrationScreen
import com.example.mobileapp.screens.UsersScreen
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun AppNavigation(){
    val navController = rememberNavController()
    Scaffold(
        bottomBar = {
            val navBackStackEntry by navController.currentBackStackEntryAsState()
            val currentRoute = navBackStackEntry?.destination?.route
            if (currentRoute != Screens.AuthorizationScreen.name && currentRoute != Screens.RegistrationScreen.name) {
                CustomNavigationBar(navController)
            }
        }
    ){
        paddingValues ->
        NavHost(
            navController = navController,
            startDestination = Screens.AuthorizationScreen.name,
            modifier = Modifier.padding(paddingValues)
        ){
            composable(route = Screens.MenuScreen.name){ MenuScreen() }
            composable(route = Screens.ProfileScreen.name){ ProfileScreen(navController) }
            composable(route = Screens.CartScreen.name){ CartScreen() }
            composable(route = Screens.ManagmentScreen.name){ ManagmentScreen(navController) }
            composable(route = Screens.OrderScreen.name){ OrderScreen() }

            composable(route = Screens.AuthorizationScreen.name){ AuthorizationScreen(navController) }
            composable(route = Screens.CreateProductScreen.name){ CreateProductScreen(navController) }
            composable(route = Screens.EditMenuScreen.name){ EditMenuScreen(navController) }
            composable(route = Screens.RegistrationScreen.name){ RegistrationScreen(navController) }
            composable(route = Screens.UsersScreen.name){ UsersScreen() }
            composable(route = Screens.EditProductScreen.name + "/{productId}"){ backStackEntry ->
                val productId = backStackEntry.arguments?.getString("productId")
                EditProductScreen(navController, productId.toString().toInt())
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun CustomNavigationBar(navController: NavController) {
    val sharedPref = PreferencesManager(LocalContext.current)
    val user_role = sharedPref.getData("userRole", "user")
    val listOfNavItems = remember { mutableStateListOf<NavItem>() }
    when (user_role) {
        "admin" -> {
            listOfNavItems.clear()
            listOfNavItems.addAll(listOfAdminNavItems)
        }
        "worker" -> {
            listOfNavItems.clear()
            listOfNavItems.addAll(listOfWorkerNavItems)
        }
        "user" -> {
            listOfNavItems.clear()
            listOfNavItems.addAll(listOfUserNavItems)
        }
    }

    Row(
        modifier = Modifier
            .fillMaxWidth()
            .fillMaxHeight(0.1f)
            .background(Color(212, 206, 203))
    ) {
        listOfNavItems.forEachIndexed { index, navItem ->
            OutlinedButton(
                modifier = Modifier
                    .fillMaxHeight()
                    .weight(1f),
                onClick = {
                    navController.navigate(navItem.route) {
                        popUpTo(navController.graph.startDestinationId)
                        launchSingleTop = true
                    }
                },
                border = BorderStroke(4.dp, Color(222,161,69)),
                shape = RoundedCornerShape(
                    topStart = if (index == 0) 30.dp else 0.dp,
                    topEnd = if (index == listOfNavItems.size - 1) 30.dp else 0.dp
                ),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),

                ){
                Icon(
                    painterResource(id = navItem.icon),
                    contentDescription = null,
                    tint = Color.Black
                )
            }
        }
    }
}