package com.example.mobileapp.screens

import android.annotation.SuppressLint
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.paging.compose.collectAsLazyPagingItems
import androidx.paging.compose.itemKey
import com.example.mobileapp.R
import com.example.mobileapp.models.OrderWithProducts
import com.example.mobileapp.viewmodels.OrderViewModel
import com.example.mobileapp.widgets.Header
import kotlinx.coroutines.launch

@Composable
fun OrderScreen(){
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        OrdersCard()
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun OrdersCard(){
    val orderViewModel: OrderViewModel = hiltViewModel<OrderViewModel>()
    val orderListUiState = orderViewModel.orderList.observeAsState().value?.collectAsLazyPagingItems()

    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,

        ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),

            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Text(
                text = "Заказы",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )
            if (orderListUiState != null){
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                ) {
                    items(
                        count = orderListUiState.itemCount,
                        key = orderListUiState.itemKey()
                    ){ index ->
                        if (orderListUiState[index] != null){
                            OrderCard(orderListUiState[index]!!, orderViewModel)
                        }
                    }
                }
            }
        }
    }
}

@SuppressLint("CoroutineCreationDuringComposition")
@Composable
fun OrderCard(order: OrderWithProducts, orderViewModel: OrderViewModel){
    val scope = rememberCoroutineScope()
    var status by remember { mutableStateOf("")}
    scope.launch {
        status = orderViewModel.getOrderStatus(order.order.statusId)
    }
    Card(
        shape = RoundedCornerShape(20.dp),
        border = BorderStroke(3.dp, Color(222,161,69)),
        modifier = Modifier
            .padding(top = 15.dp, start = 5.dp, end = 5.dp)
    ){
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceEvenly,
        ){
            Text(
                text = "№" + order.order.id,
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                textAlign = TextAlign.Center,
            )
            Text(
                text = "Статус: " + status,
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 20.sp,
                textAlign = TextAlign.Center,
            )
        }

        Row(
            modifier = Modifier
                .fillMaxWidth()
                .background(Color.White),
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceAround,
        ){
            Column {
                if (order.orderWithProducts.size != 0){
                    order.orderWithProducts.forEach{ orderProduct ->
                        Row(
                            modifier = Modifier
                                .background(Color.White),
                            verticalAlignment = Alignment.CenterVertically,
                            horizontalArrangement = Arrangement.Center,
                        ){
                            Text(
                                text = orderProduct.product.name + " x" + orderProduct.orderProduct.amount.toString(),
                                color = Color.Gray,
                                fontFamily = FontFamily(Font(R.font.nunito_bold)),
                                fontSize = 20.sp,
                                textAlign = TextAlign.Center,
                            )
                        }
                    }
                }
            }
            OutlinedButton(
                onClick = {
                    orderViewModel.changeOrderStatus(order.order.id!!)
                },
                border = BorderStroke(4.dp, Color(222,161,69)),
                shape = RoundedCornerShape(20.dp),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                modifier = Modifier.padding(bottom = 10.dp)
            ) {
                if (order.order.statusId == 2){
                    Text(
                        text = "Готов",
                        fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                        fontSize = 30.sp,
                        color = Color.White
                    )
                }
                if (order.order.statusId == 3){
                    Text(
                        text = "Выдать",
                        fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                        fontSize = 30.sp,
                        color = Color.White
                    )
                }
            }
        }
    }
}