package com.example.mobileapp.viewmodels

import androidx.compose.runtime.mutableStateListOf
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobileapp.models.OrderWithProducts
import com.example.mobileapp.models.Product
import com.example.mobileapp.models.User
import com.example.mobileapp.repositories.CategoryRepository
import com.example.mobileapp.repositories.ProductRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class ProductViewModel @Inject constructor(
    private val productRepository: ProductRepository,
    private val categoryRepository: CategoryRepository
) : ViewModel() {
    private val _successState = MutableLiveData<Boolean?>()
    private val _categories = mutableStateListOf<String>()
    val categories: List<String>
        get() = _categories
    val successState: LiveData<Boolean?>
        get() = _successState

    private val _product = MutableLiveData<Product?>()
    val product: LiveData<Product?>
        get() = _product

    fun getProduct(productId: Int) {
        viewModelScope.launch {
            _product.postValue(productRepository.getById(productId).first())
        }
    }

    init {
        viewModelScope.launch {
            categoryRepository.getAll().collect{data ->
                _categories.addAll(data.map { category -> category.name })
            }
        }
    }

    suspend fun getCategory(categoryId : Int) : String{
        return categoryRepository.getById(categoryId).first().name
    }

    fun calmSuccessState() {
        _successState.postValue(null)
    }

    fun addProduct(name: String, category: String, price: String) {
        if (name.isEmpty() || name.length > 10 || name.length < 3){
            _successState.postValue(false)
            return
        }
        if (category.isEmpty()){
            _successState.postValue(false)
            return
        }
        if (price.isEmpty() || price.toIntOrNull() == null){
            _successState.postValue(false)
            return
        }
        viewModelScope.launch {
            productRepository.insert(Product(null, name, categoryRepository.getByName(category).first().id!!, price.toInt()))
            _successState.postValue(true)
        }
    }

    fun updateProduct(id: Int, name: String, category: String, price: String) {
        if (name.isEmpty() || name.length > 10 || name.length < 3){
            _successState.postValue(false)
            return
        }
        if (category.isEmpty()){
            _successState.postValue(false)
            return
        }
        if (price.isEmpty() || price.toIntOrNull() == null){
            _successState.postValue(false)
            return
        }
        viewModelScope.launch {
            productRepository.update(Product(id, name, categoryRepository.getByName(category).first().id!!, price.toInt()))
            _successState.postValue(true)
        }
    }
}